// There are  N!/(m!(N-m)!) complete m-vertex sub graphs of a graph of N
// vertices. Subgraphs correspond to particular combinations of m remaining
// vertices. Or, alternatively to combinations of (m-n) vertices to remove.
// Here we just generate such combinations.
//
// (c) 2012 Konstantin L. Metlov <metlov@fti.dn.ua>
//
// You can redistribute and/or modify this file under the terms of the GNU
// General Public License as published by the Free Software Foundation, eith
// version 3 of the License, or (at your option) any later version.
//
// This file is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

#include <cstdlib>
#include <cmath>
#include <cassert>
#include <iostream>
#include <iomanip>
#include <bdd.h>

using namespace std;

void first_comb(char* c, int n, int k);
bool next_comb(char* c, int n, int k);

int main(int argc, char *argv[]) { 
   const int n=5;
   const int m=3;
   // print all combinations of m elements out of n elements.
   char c[m];
   first_comb(c,n,m);
   do {
     for (int i=0; i<m; i++) cerr << (int)(c[i]) << ' ';
     cerr << endl;
   } while (next_comb(c,n,m));
};

/*
   Initializes combinations generator by lexicographically smallest combination
   that is [0,1,...,k-1]. The arrat c must have at least k elements.
*/
void first_comb(char* c, int n, int k) {
   assert(k<=n);
   assert(k!=0);
   for (int i=0; i<k; i++) c[i]=i;
};

/*
   Advance to the next combination in lexicographic order.
   Algorithm from D.E. Knuth, "The art of computer programming",
   pre-fascicle 3a, section 7.2.1.3
*/
bool next_comb(char * c, int n, int k) {
   // easy case
   if (c[k-1]<n-1) {
      c[k-1]++;
      return true;
   };
   // find element to increase
   int i=k-1;
   while ((i>=0)&&(c[i]==n-k+i)) i--;
   if (i<0) return false; // reached the last combination
   c[i]++;
   for(i++;i<k;i++) c[i]=c[i-1]+1; // carry
   return true;
};
